import React from "react";
import syntaxData from "../../languages.json";
import "../styles/styling.css";

// Define the types for the JSON structure
interface LanguageRules {
  keywords?: Record<string, string[]>;
  comments?: {
    singleLine: string;
    multiLine?: string[];
  };
  strings?: {
    delimiters: string[];
  };
  numbers?: string;
  operators?: string[];
}

interface SyntaxData {
  languages: Record<string, LanguageRules>;
}

// Load the JSON file as typed data
const syntaxDataTyped: SyntaxData = syntaxData as SyntaxData;

// Define the props for the component
interface SyntaxHighlighterProps {
  code: string;
  language: string;
  theme: string;
}

const SyntaxHighlighter = ({ code, language, theme }: SyntaxHighlighterProps) => {
  const languageRules = syntaxDataTyped.languages[language];

  if (!languageRules) {
    return <pre>{code}</pre>; // Fallback if the language is not found
  }

  // Tokenize the input code string
  const tokenize = (code: string): string[] => {
    const tokenRegex = /(\".*?\"|'.*?'|`.*?`|\/\/.*?$|\/\*[\s\S]*?\*\/|[\w$]+|[+\-*/=%!&|^~<>?:;.,{}[\]()\s]|[^\s])/gm;
    return code.match(tokenRegex) || [];
  };

  // Determine the type of a token
  const getTokenType = (token: string): string => {
    // Check if the token matches any keywords
    if (languageRules.keywords) {
      for (const [category, keywords] of Object.entries(languageRules.keywords)) {
        if (keywords.includes(token)) {
          return `keyword-${category}`;
        }
      }
    }

    // Check if it's an operator
    if (languageRules.operators?.includes(token)) {
      return "operator";
    }

    // Check if it's a number
    if (languageRules.numbers && new RegExp(languageRules.numbers).test(token)) {
      return "number";
    }

    // Check if it's a string
    if (languageRules.strings?.delimiters.some((d: string) => token.startsWith(d))) {
      return "string";
    }

    // Check if it's a comment
    if (
      token.startsWith(languageRules.comments?.singleLine || "") ||
      (languageRules.comments?.multiLine &&
        token.startsWith(languageRules.comments.multiLine[0]) &&
        token.endsWith(languageRules.comments.multiLine[1]))
    ) {
      return "comment";
    }

    return "identifier"; // Default to identifier
  };

  const formatCode = (code: string): string => {
    let indentLevel = 0;
    const lines = code.split("\n");
    const formattedLines = lines.map((line) => {
      line = line.trim();

      if (line === "") {
        return "\n"; // Keep empty lines
      }

      const indentation = "  ".repeat(indentLevel);

      // Adjust indent level for braces
      if (line.startsWith("}")) {
        indentLevel = Math.max(indentLevel - 1, 0);
      }

      // Format the current line
      const formattedLine = `${indentation}${line}`;

      // Adjust indent level for opening braces
      if (line.endsWith("{") && !line.startsWith("catch")) {
        indentLevel++;
      }

      // Special handling for `catch` to align it with `try`
      if (line.startsWith("catch")) {
        indentLevel = Math.max(indentLevel - 1, 0);
      }

      // Adjust for closing braces at the end of the line
      if (line.endsWith("}") && !line.startsWith("}")) {
        indentLevel = Math.max(indentLevel - 1, 0);
      }

      return formattedLine;
    });

    // Remove extra empty lines
    return formattedLines.join("\n").replace(/\n{2,}/g, "\n");
  };

  // Highlight the code
  const highlightCode = (code: string): React.ReactNode[] => {
    const formattedCode = formatCode(code);
    const tokens = tokenize(formattedCode);
    return tokens.map((token, index) => {
      const type = getTokenType(token);

      return (
        <span key={index} className={`token ${type}`}>
          {token === "\t" ? <span style={{ marginLeft: "1em" }}></span> : token}
        </span>
      );
    });
  };

  return (
    <pre className={`syntax-highlighter ${language} ${theme}`}>
      <code>{highlightCode(code)}</code>
    </pre>
  );
};

export default SyntaxHighlighter;
